<?php

class NodeIterator
{
 function hasNext() {}
 function &next() {}
 function remove() {}
 function getKey() {}
}

class ObjectIterator extends NodeIterator
{
 var $d_object;
 var $d_pos=0;
 var $d_length;

 function ObjectIterator( &$object )
 {
 $this->d_object = &$object; 
 $this->d_length = $object->size();
 }
 
 function hasNext()
 {
 return $this->d_pos < $this->d_length;
 }
 
 function remove()
 {
 if ( $this->d_pos == 0 ) return;
 $obj = &$this->d_object->remove( $this->getKey() );
 $this->d_length--;
 $this->d_pos--;
 }
 
 function &next()
 {
 if ( !$this->hasNext() )
 return NULL;
 $ret =& $this->d_object->getObjectRef( "collobj".$this->d_pos );
 $this->d_pos++;
 return $ret;
 }

 function getKey() {
 return $this->d_pos - 1;
 }

}

class HashObjectIterator extends ObjectIterator {

 var $d_keys;

 function HashObjectIterator( &$object, &$keys ) {
 parent::ObjectIterator($object);
 $this->d_keys = array_flip($keys);
 }

 function getKey() {
 return $this->d_keys[$this->d_pos - 1];
 }
}


class ReverseObjectIterator extends NodeIterator
{
 var $d_object;
 var $d_pos;
 var $d_length;

 function ReverseObjectIterator( &$object )
 {
 $this->d_objects = &$object; 
 $this->d_length = $object->size();
 $this->d_pos = $this->d_length-1;
 }
 
 function hasNext()
 {
 return $this->d_pos >= 0;
 }
 
 function remove()
 {
 if ( $this->d_pos == $this->d_length-1 ) return;
 $this->d_object->remove( $this->getKey() );
 $this->d_length--;
 }
 
 function &next()
 {
 if ( !$this->hasNext() )
 return NULL;
 $ret = &$this->d_object->getObjectRef( "collobj".$this->d_pos );
 $this->d_pos--;
 return $ret;
 }

 function getKey() {
 return $this->d_pos + 1;
 }

}

class ReverseHashObjectIterator extends ReverseObjectIterator {

 var $d_keys;

 function ReverseHashObjectIterator( &$object, &$keys ) {
 parent::ReverseObjectIterator($object);
 $this->d_keys = array_flip($keys);
 }

 function getKey() {
 return $this->d_keys[$this->d_pos + 1];
 }
}


class Collection extends BaseObject
{
 
 var $d_objCount = 0;
 
 function Collection() {
 parent::BaseObject();
 }
 
 function add( &$object )
 {
 if ( !is_a( $object, "baseobject" ) )
 return FALSE;
 $this->setObjectRef( "collobj".$this->d_objCount, $object );
 $this->d_objCount++;
 return $this->d_objCount-1; 
 }

 function &get( $index )
 {
 if ( $index < 0 || $index >= $this->d_objCount ) return NULL;
 return $this->getObjectRef( "collobj".$index );
 }
 
 function& remove( $index )
 {
 if ( $index < 0 || $index >= $this->d_objCount ) return NULL;
 $ret =& $this->getObjectRef( "collobj".$index );
 for ($i = $index; $i < $this->d_objCount - 1; $i++) {
 $this->setObjectRef( "collobj".$i, $this->getObjectRef( "collobj".($i+1) ) );
 }
 $this->unsetObjectRef( "collobj".$i );
 $this->d_objCount--;
 return $ret;
 }
 
 function addAll( &$objArray )
 {
 foreach ( $objArray as $key=>$val )
 {
 $this->add( $objArray[$key] );
 }
 }
 
 function clear() {
 for ( $i = 0; $i < $this->d_objCount; $i++ )
 {
 $this->unsetObjectRef( "collobj".$i );
 }
 $this->d_objCount = 0;
 }
 
 function size()
 {
 return $this->d_objCount;
 }
 
 function iterator()
 {
 return new ObjectIterator( $this );
 }

 function reverseIterator()
 {
 return new ReverseObjectIterator( $this );
 }
 
 function toString()
 {
 $it = $this->iterator();
 $ret = "Collection size(".$this->size()."):<br />";
 while ( $it->hasNext() )
 {
 $next = $it->next();
 $ret .= $next->toString() . "<br />";
 }
 return $ret;
 }

 function toArray() {
 return $this->d_objs;
 }
}

class HashCollection extends Collection {

 var $d_keyMap;

 function HashCollection() {
 parent::Collection();
 $this->d_keyMap = array();
 }

 function add( $key, &$object ) {
 $index = parent::add( $object );
 $this->d_keyMap[$key] = $index;
 
 return $index;
 }

 function insert( $key, &$object, $afterKey ) {
 
 if (($afterKey && !isset($this->d_keyMap[$afterKey])) || count($this->d_keyMap) == 0) {
 $this->add($key, $object);
 return false;
 }
 
 $idx = $afterKey ? $this->d_keyMap[$afterKey] + 1 : 0 ;
 foreach ($this->d_keyMap as $k=>$i) {
 if ($i >= $idx) $this->d_keyMap[$k]++;
 }
 for ($i = count($this->d_keyMap) - 1; $i >= $idx; $i--) {
 $this->setObjectRef( "collobj".($i + 1), $this->getObjectRef( "collobj".$i ) );
 }
 $this->setObjectRef( "collobj".$idx, $object );
 $this->d_keyMap[$key] = $idx;
 $this->d_objCount++;
 asort($this->d_keyMap);
 
 return ($idx > 0);
 }

 function addAll( &$objArray ) {
 
 foreach($objArray as $key => $obj) {
 $this->add( $key, $objArray[$key] );
 }
 }

 function &remove( $key )
 {
 if ( !isset( $this->d_keyMap[$key] ) ) return NULL;
 $index = $this->d_keyMap[$key];
 
 $r = &parent::remove( $index );
 unset( $this->d_keyMap[$key] );
 foreach ( $this->d_keyMap as $key=>$idx )
 {
 if ( $idx >= $index )
 {
 $this->d_keyMap[$key]--;
 }
 }
 return $r;
 }
 
 function clear() {
 parent::clear();
 $this->d_keyMap = array();
 }

 
 function iterator() {
 return new HashObjectIterator( $this, $this->d_keyMap );
 }

 function reverseIterator() {
 return new ReverseHashObjectIterator( $this, $this->d_keyMap );
 }
 
 function& getByKey( $key )
 {
 
 if ( isset( $this->d_keyMap[$key] ) )
 return $this->get( $this->d_keyMap[$key] );
 return new NodeException( "HashCollection: key ($key) not defined" );
 }

 function keyExists( $key )
 {
 if ( isset( $this->d_keyMap[$key] ) )
 return TRUE;
 return FALSE;
 }
 
 function getKeys() {
 return array_keys( $this->d_keyMap );
 }

 function getPropertyNames() {
 return $this->getKeys();
 }

 function &getProperty( $key ) {
 return $this->getByKey($key);
 }

}

class NodeObjectHashCollection extends HashCollection 
{
 var $d_TType;
 var $d_listHeaders; 

 






 function NodeObjectHashCollection ( $TType, $listHeaders=array() )
 {
 parent::HashCollection();
 $this->d_TType = $TType; 
 $this->d_listHeaders = $listHeaders;
 }
 
 function getCollectedTType()
 {
 return $this->d_TType;
 }
 
 function getListHeaders() 
 {
 return $this->d_listHeaders;
 }
 
 function getIcon() {}

 function add( $key, &$object )
 {
 if ( !is_a( $object, "nodeobject" ) ) {
 $this->addDebugMessage( "NodeObjectCollection: rejecting object: ".get_class($object) );
 return FALSE;
 }
 if ( $object->getTType() != $this->d_TType )
 {
 $this->addDebugMessage( "NodeObjectCollection: rejecting t-type: ".$object->getTType() );
 return FALSE;
 }
 return parent::add( $key, $object );
 }
 
 









 
 
 
 

 







 function getValueFields() {
 return NULL;
 }

 






 function &getFormObject() { return new NodeCollectionFormObject( $this ); }

 





 function checkErrors() { }

 function canInsert() { return false; }

 function canUpdate() { return true; }

 function canDelete() { return false; }
 function isNew() { return false; }

 function isInitialized() { return false; }

 function isModified() { return false; }

 function create() { } 
 
 function save() { }

 function delete() { }

 function getErrors() { }

 function resetErrors() { } 
}

class NodeObjectCollection extends NodeObjectHashCollection {
 
 function NodeObjectCollection( $TType, $listHeaders=array() ) {
 parent::NodeObjectHashCollection( $TType, $listHeaders );
 }
 
 function add( &$object ) {
 if (!is_a($object, "nodeobject")) {
 $this->addDebugMessage( "NodeObjectCollection: rejecting object: ".get_class($object) );
 return false;
 }
 return parent::add( $object->getObjectKey(), $object );
 }
}

class NodeCollectionFormObject extends NodeFormObject {

 function NodeCollectionFormObject ( &$part ) {
 parent::NodeFormObject( $part );
 }

 function isNew() {
 return false;
 }
 
 function getValueFields() {
 $obj =& $this->getEditObject();
 $it =& $obj->iterator();
 $ret = array();
 while ( $it->hasNext() )
 {
 $it->next();
 $ret[$it->getKey()] = false;
 }
 return $ret;
 }

 
 
 






 function getListHeaders() { return array(); }
 
 


 function getFieldDescriptions() { return array(); }

 function getFieldLabels() {
 return array(); 
 }
 
 




 function isEditableField($field) { 
 return true;
 }

 function isOptionalField($field) { 
 return true;
 }

 




 function &getFormComponent($field) {
 $obj =& $this->getEditObject();
 $nodeObject =& $obj->getByKey( $field );
 $nodeObjectForm =& $nodeObject->getFormObject();
 $ret =& $nodeObjectForm->getCollectionComponent();
 return $ret;
 }

 


 function storeFormComponent($field, &$component) { 
 $obj = &$this->getEditObject();
 $nodeObject =& $obj->getByKey( $field );
 $nodeObjectForm =& $nodeObject->getFormObject();
 $nodeObjectForm->storeCollectionComponent($component);
 }
}



?>
